@@ -1,4 +1,4 @@
-/* $Id: campaign_server.cpp 33338 2009-03-04 12:25:00Z silene $ */
+/* $Id: campaign_server.cpp 35864 2009-05-24 09:54:26Z soliton $ */
 /*
    Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -503,9 +503,9 @@ namespace {
 						} else if(((*upload)["title"]).empty()) {
 							LOG_CS << "Upload aborted - no add-on title specified.\n";
 							network::send_data(construct_error("Add-on rejected: You did not specify the title of the add-on in the pbl file!"), sock, gzipped);
-						} else if(((*upload)["type"]).empty()) {
-							LOG_CS << "Upload aborted - no add-on type specified.\n";
-							network::send_data(construct_error("Add-on rejected: You did not specify the type of the add-on in the pbl file!"), sock, gzipped);
+						} else if (get_addon_type((*upload)["type"]) == ADDON_UNKNOWN) {
+							LOG_CS << "Upload aborted - unknown add-on type specified.\n";
+							network::send_data(construct_error("Add-on rejected: You did not specify a known type for the add-on in the pbl file! (See PblWML: wiki.wesnoth.org/PblWML)"), sock, gzipped);
 						} else if(((*upload)["author"]).empty()) {
 							LOG_CS << "Upload aborted - no add-on author specified.\n";
 							network::send_data(construct_error("Add-on rejected: You did not specify the author(s) of the add-on in the pbl file!"), sock, gzipped);
