@@ -1,4 +1,4 @@
-/* $Id: font.cpp 33687 2009-03-15 16:50:42Z mordante $ */
+/* $Id: font.cpp 37708 2009-08-13 10:03:28Z loonycyborg $ */
 /* vim:set encoding=utf-8: */
 /*
    Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
@@ -326,7 +326,7 @@ void manager::init() const
 		std::vector<std::string> files;
 		get_files_in_dir(path, &files, NULL, ENTIRE_FILE_PATH);
 		foreach(const std::string& file, files)
-			if(file.substr(file.length() - 4) == ".ttf")
+			if(file.substr(file.length() - 4) == ".ttf" || file.substr(file.length() - 4) == ".ttc")
 				AddFontResource(file.c_str());
 	}
 #endif
@@ -343,7 +343,7 @@ void manager::deinit() const
 		std::vector<std::string> files;
 		get_files_in_dir(path, &files, NULL, ENTIRE_FILE_PATH);
 		foreach(const std::string& file, files)
-			if(file.substr(file.length() - 4) == ".ttf")
+			if(file.substr(file.length() - 4) == ".ttf" || file.substr(file.length() - 4) == ".ttc")
 				RemoveFontResource(file.c_str());
 	}
 #endif
@@ -677,17 +677,16 @@ static surface render_text(const std::string& text, int fontsize, const SDL_Colo
 
 	for(std::vector< std::string >::const_iterator ln = lines.begin(), ln_end = lines.end(); ln != ln_end; ++ln) {
 
-		SDL_Color col = colour;
 		int sz = fontsize;
 		int text_style = style;
 
 		std::string::const_iterator after_markup = use_markup ?
-			parse_markup(ln->begin(),ln->end(),&sz,&col,&text_style) : ln->begin();
-		text_surface txt_surf(sz, col, text_style);
+			parse_markup(ln->begin(), ln->end(), &sz, NULL, &text_style) : ln->begin();
+		text_surface txt_surf(sz, colour, text_style);
 
-		if (after_markup == ln->end() && ln+1 != ln_end) {
+		if (after_markup == ln->end() && (ln+1 != ln_end || lines.begin()+1 == ln_end)) {
 			// we replace empty line by a space (to have a line height)
-			// except for the blank following the last '\n'
+			// except for the last line if we have several
 			txt_surf.set_text(" ");
 		} else if (after_markup == ln->begin()) {
 		 	// simple case, no markup to skip
@@ -1252,7 +1251,7 @@ static bool add_font_to_fontlist(config* fonts_config, std::vector<font::subset_
 namespace font {
 
 namespace {
-	t_string font_order;
+	t_string family_order;
 } // namespace
 
 bool load_font_config()
@@ -1279,7 +1278,7 @@ bool load_font_config()
 		known_fonts.insert((**child)["name"]);
 	}
 
-	font_order = (*fonts_config)["order"];
+	family_order = (*fonts_config)["family_order"];
 	const std::vector<std::string> font_order = utils::split((*fonts_config)["order"]);
 	std::vector<font::subset_descriptor> fontlist;
 	std::vector<std::string>::const_iterator font;
@@ -1299,9 +1298,9 @@ bool load_font_config()
 	return true;
 }
 
-const t_string& get_fonts()
+const t_string& get_font_families()
 {
-	return font_order;
+	return family_order;
 }
 
 void cache_mode(CACHE mode)
