@@ -453,6 +453,11 @@ game_controller::game_controller(int argc, char** argv) :
 	else if (no_music) { // else disable the music in nomusic mode
 		preferences::set_music(false);
 	}
+
+#ifdef MAEMO_CHANGES
+	game_config::small_gui = true;
+	force_bpp_ = 16;
+#endif
 }
 
 game_display& game_controller::disp()
