@@ -1,4 +1,4 @@
-/* $Id: game.cpp 33600 2009-03-13 13:23:48Z soliton $ */
+/* $Id: game.cpp 38325 2009-09-02 16:40:00Z soliton $ */
 /*
    Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -14,17 +14,25 @@
 
 #include "../global.hpp"
 
+#include "../filesystem.hpp"
 #include "../game_config.hpp" // game_config::observer_team_name
 #include "../log.hpp"
 #include "../map.hpp" // gamemap::MAX_PLAYERS
 
 #include "game.hpp"
 
+#ifndef __func__
+ #ifdef __FUNCTION__
+  #define __func__ __FUNCTION__
+ #endif
+#endif
+
 
 #define ERR_GAME LOG_STREAM(err, mp_server)
 #define WRN_GAME LOG_STREAM(warn, mp_server)
 #define LOG_GAME LOG_STREAM(info, mp_server)
 #define DBG_GAME LOG_STREAM(debug, mp_server)
+#define WRN_CONFIG LOG_STREAM(warn, config)
 
 namespace chat_message {
 
@@ -47,7 +55,8 @@ namespace wesnothd {
 int game::id_num = 1;
 
 game::game(player_map& players, const network::connection host,
-		const std::string name) :
+		const std::string name, bool save_replays,
+		const std::string replay_save_path) :
 	player_info_(&players),
 	id_(id_num++),
 	name_(name),
@@ -66,7 +75,9 @@ game::game(player_map& players, const network::connection host,
 	end_turn_(0),
 	all_observers_muted_(false),
 	bans_(),
-	termination_()
+	termination_(),
+	save_replays_(save_replays),
+	replay_save_path_(replay_save_path)
 {
 	// Hack to handle the pseudo games lobby_ and not_logged_in_.
 	if (owner_ == 0) return;
@@ -83,9 +94,16 @@ game::game(player_map& players, const network::connection host,
 
 game::~game()
 {
-	for(std::vector<simple_wml::document*>::iterator i = history_.begin(); i != history_.end(); ++i) {
-		delete *i;
+	// Hack to handle the pseudo games lobby_ and not_logged_in_.
+	if (owner_ == 0) return;
+
+	save_replay();
+
+	user_vector users = all_game_users();
+	for (user_vector::const_iterator u = users.begin(); u != users.end(); ++u) {
+		remove_player(*u, false, true);
 	}
+	clear_history();
 }
 
 bool game::allow_observers() const {
@@ -150,6 +168,8 @@ void game::start_game(const player_map::const_iterator starter) {
 			"\tturn bonus: "     + s["mp_countdown_turn_bonus"].to_string() : "")
 		<< "\n";
 
+	update_side_data();
+
 	nsides_ = 0;
 	// Set all side controllers to 'human' so that observers will understand
 	// that they can't take control of any sides if they happen to have the
@@ -157,7 +177,14 @@ void game::start_game(const player_map::const_iterator starter) {
 	const simple_wml::node::child_list& sides = level_.root().children("side");
 	for(simple_wml::node::child_list::const_iterator s = sides.begin(); s != sides.end(); ++s) {
 		nsides_++;
-		if((**s)["controller"] != "null" && !advance) {
+		if ((**s)["controller"] != "null") {
+			int side_num = (**s)["side"].to_int() - 1;
+			if (sides_[side_num] == 0) {
+				std::stringstream msg;
+				msg << "Side "  << side_num + 1 << " has no controller but should! The host needs to assign control for the game to proceed past that side's turn.";
+				LOG_GAME << msg.str() << " (game id: " << id_ << ")\n";
+				send_and_record_server_message(msg.str().c_str());
+			}
 			(*s)->set_attr("controller", "human");
 		}
 	}
@@ -174,10 +201,8 @@ void game::start_game(const player_map::const_iterator starter) {
 	}
 	end_turn_ = (turn - 1) * nsides_ + side - 1;
 	end_turn();
-	history_.clear();
+	clear_history();
 	if (advance) {
-		// Re-assign sides to allow correct filtering of commands.
-		update_side_data();
 		// When the host advances tell everyone that the next scenario data is
 		// available.
 		static simple_wml::document notify_next_scenario("[notify_next_scenario]\n[/notify_next_scenario]\n", simple_wml::INIT_COMPRESSED);
@@ -187,10 +212,21 @@ void game::start_game(const player_map::const_iterator starter) {
 	send_observerjoins();
 }
 
+bool game::send_taken_side(simple_wml::document& cfg, const simple_wml::node::child_list::const_iterator side) const
+{
+	const size_t side_num = (**side)["side"].to_int();
+	if (side_num < 1 || side_num > gamemap::MAX_PLAYERS) return false;
+	if (sides_[side_num - 1] != 0) return false;
+	// We expect that the host will really use our proposed side number. (He could do different...)
+	cfg.root().set_attr_dup("side", (**side)["side"]);
+
+	// Tell the host which side the new player should take.
+	return send_to_one(cfg, owner_);
+}
+
 bool game::take_side(const player_map::const_iterator user)
 {
 	DBG_GAME << "take_side...\n";
-	DBG_GAME << debug_player_info();
 
 	if (started_) return false;
 
@@ -200,7 +236,6 @@ bool game::take_side(const player_map::const_iterator user)
 	cfg.root().set_attr("leader", "random");
 	cfg.root().set_attr("gender", "random");
 
-	size_t side_num;
 	// Check if we can figure out a fitting side.
 	const simple_wml::node::child_list& sides = level_.root().children("side");
 	for(simple_wml::node::child_list::const_iterator side = sides.begin(); side != sides.end(); ++side) {
@@ -208,40 +243,13 @@ bool game::take_side(const player_map::const_iterator user)
 				&& ((**side)["save_id"] == user->second.name().c_str()
 				|| (**side)["current_player"] == user->second.name().c_str()))
 		{
-			side_num = (**side)["side"].to_int();
-			if (side_num < 1 || side_num > gamemap::MAX_PLAYERS) continue;
-			if (sides_[side_num - 1] != 0) continue;
-			side_controllers_[side_num - 1] = "human";
-			sides_[side_num - 1] = user->first;
-			cfg.root().set_attr_dup("side", (**side)["side"]);
-			// Tell the host which side the new player should take.
-
-			simple_wml::string_span data = cfg.output_compressed();
-			network::send_raw_data(data.begin(), data.size(), owner_, cfg.root().first_child().to_string());
-			DBG_GAME << "take_side: took side " << side_num << " because the name matched\n";
-			DBG_GAME << debug_player_info();
-			return true;
+			if (send_taken_side(cfg, side)) return true;
 		}
 	}
 	// If there was no fitting side just take the first available.
 	for(simple_wml::node::child_list::const_iterator side = sides.begin(); side != sides.end(); ++side) {
 		if((**side)["controller"] == "network") {
-			//don't allow players to take sides in games with invalid side numbers
-			try {
-				side_num = (**side)["side"].to_int();
-			} catch (bad_lexical_cast&) { continue; }
-			if (side_num < 1 || side_num > gamemap::MAX_PLAYERS) continue;
-			if (sides_[side_num - 1] != 0) continue;
-			// we expect that the host will really use our proposed side number (he could do different)
-			side_controllers_[side_num - 1] = "human";
-			sides_[side_num - 1] = user->first;
-			cfg.root().set_attr_dup("side", (**side)["side"]);
-			// Tell the host which side the new player should take.
-			simple_wml::string_span data = cfg.output_compressed();
-			network::send_raw_data(data.begin(), data.size(), owner_, cfg.root().first_child().to_string());
-			DBG_GAME << "take_side: took the first free network side which was " << side_num << "\n";
-			DBG_GAME << debug_player_info();
-			return true;
+			if (send_taken_side(cfg, side)) return true;
 		}
 	}
 	DBG_GAME << "take_side: there are no more sides available\n";
@@ -294,7 +302,8 @@ void game::update_side_data() {
 					sides_[side_num] = *user;
 					side_found = true;
 				}
-			} else if ((**side)["controller"] == "ai" || (**side)["controller"] == "human") {
+			} else if (*user == owner_
+			&& ((**side)["controller"] == "ai" || (**side)["controller"] == "human")) {
 				side_controllers_[side_num] = (**side)["controller"].to_string();
 				sides_[side_num] = owner_;
 				side_found = true;
@@ -319,12 +328,6 @@ void game::transfer_side_control(const network::connection sock, const simple_wm
 		return;
 	}
 
-	//check, if this socket belongs to a player
-	const user_vector::iterator pl = std::find(players_.begin(), players_.end(), sock);
-	if (pl == players_.end()) {
-		ERR_GAME << "ERROR: Not a player of this game. (socket: " << sock << ")\n";
-		return;
-	}
 	// Check the side number.
 	const unsigned int side_num = cfg["side"].to_int();
 	if(side_num < 1 || side_num > gamemap::MAX_PLAYERS) {
@@ -344,10 +347,10 @@ void game::transfer_side_control(const network::connection sock, const simple_wm
 	const network::connection old_player = sides_[side_num - 1];
 	const player_map::const_iterator oldplayer = player_info_->find(old_player);
 	if (oldplayer == player_info_->end()) {
-		ERR_GAME << "ERROR: Could not find player in player_info_. (socket: "
+		WRN_GAME << "Could not find old player in player_info_. (socket: "
 			<< old_player << ")\n";
-		return;
 	}
+	const std::string old_player_name(oldplayer != player_info_->end() ? oldplayer->second.name() : "(unknown)");
 
 	// A player (un)droids his side.
 	if (newplayer_name.empty()) {
@@ -365,11 +368,10 @@ void game::transfer_side_control(const network::connection sock, const simple_wm
 			send_server_message(msg.str().c_str(), sock);
 			return;
 		}
-		change_controller(side_num - 1, oldplayer, false, cfg["controller"].to_string());
+		change_controller(side_num - 1, old_player, old_player_name, false, cfg["controller"].to_string());
 		return;
 	}
 
-	const std::string old_player_name = oldplayer->second.name();
 	// Check if the sender actually owns the side he gives away or is the host.
 	if (!(sock == old_player || sock == owner_)) {
 		std::stringstream msg;
@@ -388,7 +390,7 @@ void game::transfer_side_control(const network::connection sock, const simple_wm
 	}
 	// Is he in this game?
 	if (newplayer == player_info_->end() || !is_member(newplayer->first)) {
-		send_server_message("Player/Observer not in this game", sock);
+		send_server_message((newplayer_name.to_string() + " is not in this game").c_str(), sock);
 		return;
 	}
 
@@ -399,9 +401,9 @@ void game::transfer_side_control(const network::connection sock, const simple_wm
 		return;
 	}
 	sides_[side_num - 1] = 0;
-	bool host_leave = false;
 	// If the old player lost his last side, make him an observer.
-	if (std::find(sides_.begin(), sides_.end(), old_player) == sides_.end()) {
+	if (std::find(sides_.begin(), sides_.end(), old_player) == sides_.end()
+	&& is_player(old_player)) {
 		observers_.push_back(old_player);
 		players_.erase(std::remove(players_.begin(), players_.end(), old_player), players_.end());
 		// Tell others that the player becomes an observer.
@@ -410,20 +412,8 @@ void game::transfer_side_control(const network::connection sock, const simple_wm
 		simple_wml::document observer_join;
 		observer_join.root().add_child("observer").set_attr_dup("name", old_player_name.c_str());
 		send_data(observer_join, old_player);
-		// If the old player was the host of the game, choose another player.
-		/*if (old_player == owner_) {
-			host_leave = true;
-			if (players_.empty()) {
-				owner_ = newplayer->first;
-			} else {
-				owner_ = players_.front();
-			}
-			notify_new_host();
-		}*/
 	}
-	change_controller(side_num - 1, newplayer, false);
-
-	if (host_leave) transfer_ai_sides();
+	change_controller(side_num - 1, newplayer->first, newplayer->second.name(), false);
 
 	// If we gave the new side to an observer add him to players_.
 	if (is_observer(newplayer->first)) {
@@ -435,19 +425,19 @@ void game::transfer_side_control(const network::connection sock, const simple_wm
 }
 
 void game::change_controller(const size_t side_num,
-		const player_map::const_iterator player,
+		const network::connection sock,
+		const std::string& player_name,
 		const bool player_left,
-		const std::string controller)
+		const std::string& controller)
 {
 	DBG_GAME << __func__ << "...\n";
-	if (player == player_info_->end()) return;
 
-	const network::connection sock = player->first;
-	const std::string& player_name = player->second.name();
 	const std::string& side = lexical_cast<std::string, size_t>(side_num + 1);
 	sides_[side_num] = sock;
 
-	if (controller.empty()) {
+	if (player_left && side_controllers_[side_num] == "ai") {
+		// Automatic AI side transfer.
+	} else if (controller.empty()) {
 		send_and_record_server_message((player_name
 				+ " takes control of side " + side + ".").c_str());
 		side_controllers_[side_num] = "human";
@@ -485,27 +475,6 @@ void game::change_controller(const size_t side_num,
 	}
 }
 
-void game::transfer_ai_sides() {
-	bool ai_transfer = false;
-	// Check for ai sides first and drop them, too, if the host left.
-	for (size_t side = 0; side < side_controllers_.size(); ++side){
-		//send the host a notification of removal of this side
-		if (side_controllers_[side] != "ai") continue;
-
-		ai_transfer = true;
-		simple_wml::document drop;
-		const std::string side_drop = lexical_cast<std::string, size_t>(side + 1);
-		drop.root().set_attr("side_drop", side_drop.c_str());
-		drop.root().set_attr("controller", "ai");
-		const simple_wml::string_span data = drop.output_compressed();
-		network::send_raw_data(data.begin(), data.size(), owner_, drop.root().first_child().to_string());
-		sides_[side] = owner_;
-	}
-	if (ai_transfer) {
-		send_and_record_server_message("AI sides transferred to new host.");
-	}
-}
-
 void game::notify_new_host(){
 	const std::string owner_name =
 			(player_info_->find(owner_) != player_info_->end()
@@ -516,10 +485,11 @@ void game::notify_new_host(){
 	// Why do we send the new host his own name?
 	cfg_host_transfer.set_attr("name", owner_name.c_str());
 	cfg_host_transfer.set_attr("value", "1");
-	const simple_wml::string_span data = cfg.output_compressed();
-	network::send_raw_data(data.begin(), data.size(), owner_,cfg.root().first_child().to_string());
-	send_and_record_server_message((owner_name
-			+ " has been chosen as the new host.").c_str());
+	std::string message = owner_name + " has been chosen as the new host.";
+	if (!send_to_one(cfg, owner_)) {
+		message += " But an internal error occured. You probably have to abandon this game.";
+	}
+	send_and_record_server_message(message.c_str());
 }
 
 bool game::describe_slots() {
@@ -581,7 +551,8 @@ void game::mute_observer(const simple_wml::node& mute, const player_map::const_i
 			if (muted_nicks != "") {
 				muted_nicks += ", ";
 			}
-			muted_nicks += player_info_->find(*muted_obs)->second.name();
+			muted_nicks += (player_info_->find(*muted_obs) != player_info_->end()
+					? player_info_->find(*muted_obs)->second.name() : "");
 		}
 
 		send_server_message(("Muted observers: " + muted_nicks).c_str(), muter->first);
@@ -608,11 +579,18 @@ void game::mute_observer(const simple_wml::node& mute, const player_map::const_i
 	}
 	muted_observers_.push_back(user->first);
 	LOG_GAME << network::ip_address(muter->first) << "\t"
-		<< muter->second.name() << " muted: " << user->second.name()
-		<< "\tfrom game:\t\"" << name_ << "\" (" << id_ << ")\n";
+		<< muter->second.name() << " muted: " << name << " ("
+		<< network::ip_address(user->first) << ")\tin game:\t\""
+		<< name_ << "\" (" << id_ << ")\n";
 	send_and_record_server_message((user->second.name() + " has been muted.").c_str());
 }
 
+void game::send_leave_game(network::connection user) const
+{
+	static simple_wml::document leave_game("[leave_game]\n[/leave_game]\n", simple_wml::INIT_COMPRESSED);
+	send_to_one(leave_game, user);
+}
+
 network::connection game::kick_member(const simple_wml::node& kick,
 		const player_map::const_iterator kicker)
 {
@@ -625,20 +603,21 @@ network::connection game::kick_member(const simple_wml::node& kick,
 	if (user == player_info_->end() || !is_member(user->first)) {
 		send_server_message("Not a member of this game.", kicker->first);
 		return 0;
-	}
-	if (user->first == kicker->first) {
+	} else if (user->first == kicker->first) {
 		send_server_message("Don't kick yourself, silly.", kicker->first);
 		return 0;
+	} else if (user->second.is_moderator()) {
+		send_server_message("You're not allowed to kick a moderator.", kicker->first);
+		return 0;
 	}
 	LOG_GAME << network::ip_address(kicker->first) << "\t"
-		<< kicker->second.name() << "\tkicked: " << user->second.name()
-		<< "\tfrom game:\t\"" << name_ << "\" (" << id_ << ")\n";
+		<< kicker->second.name() << "\tkicked: " << name << " ("
+		<< network::ip_address(user->first) << ")\tfrom game:\t\""
+		<< name_ << "\" (" << id_ << ")\n";
 	send_and_record_server_message((name.to_string() + " has been kicked.").c_str());
 
 	// Tell the user to leave the game.
-	static simple_wml::document leave_game("[leave_game]\n[/leave_game]\n", simple_wml::INIT_COMPRESSED);
-	static const simple_wml::string_span leave_game_data = leave_game.output_compressed();
-	network::send_raw_data(leave_game_data.begin(), leave_game_data.size(), user->first,leave_game.root().first_child().to_string());
+	send_leave_game(user->first);
 	remove_player(user->first);
 	return user->first;
 }
@@ -655,27 +634,25 @@ network::connection game::ban_user(const simple_wml::node& ban,
 	if (user == player_info_->end()) {
 		send_server_message("User not found", banner->first);
 		return 0;
-	}
-	if (user->first == banner->first) {
+	} else if (user->first == banner->first) {
 		send_server_message("Don't ban yourself, silly.", banner->first);
 		return 0;
-	}
-	if (player_is_banned(user->first)) {
-		std::ostringstream stream;
-		stream << name << " is already banned.";
-		send_server_message(stream.str().c_str(), banner->first);
+	} else if (player_is_banned(user->first)) {
+		send_server_message(("'" + name.to_string() + "' is already banned.").c_str(), banner->first);
+		return 0;
+	} else if (user->second.is_moderator()) {
+		send_server_message("You're not allowed to ban a moderator.", banner->first);
 		return 0;
 	}
 	LOG_GAME << network::ip_address(banner->first) << "\t"
-		<< banner->second.name() << "\tbanned: " << name << "\tfrom game:\t"
+		<< banner->second.name() << "\tbanned: " << name << " ("
+		<< network::ip_address(user->first) << ")\tfrom game:\t\""
 		<< name_ << "\" (" << id_ << ")\n";
 	bans_.push_back(network::ip_address(user->first));
 	send_and_record_server_message((name.to_string() + " has been banned.").c_str());
 	if (is_member(user->first)) {
 		//tell the user to leave the game.
-		static simple_wml::document leave_game("[leave_game]\n[/leave_game]\n", simple_wml::INIT_COMPRESSED);
-		static const simple_wml::string_span leave_game_data = leave_game.output_compressed();
-		network::send_raw_data(leave_game_data.begin(), leave_game_data.size(), user->first,leave_game.root().first_child().to_string());
+		send_leave_game(user->first);
 		remove_player(user->first);
 		return user->first;
 	}
@@ -727,8 +704,7 @@ bool game::is_legal_command(const simple_wml::node& command, bool is_player) {
 		|| command.child("clear_labels")
 		|| command.child("rename")
 		|| command.child("countdown_update")
-		|| command.child("advance_unit")
-		|| command.child("choose")))
+		))
 	{
 		return true;
 	}
@@ -748,21 +724,24 @@ bool game::process_turn(simple_wml::document& data, const player_map::const_iter
 	std::vector<int> marked;
 	const simple_wml::node::child_list& commands = turn->children("command");
 	simple_wml::node::child_list::const_iterator command;
+	const bool player = (is_player(user->first) || user->first == owner_);
 	for (command = commands.begin(); command != commands.end(); ++command) {
 		if (!is_current_player(user->first)
-		&& !is_legal_command(**command, is_player(user->first))) {
-			LOG_GAME << "ILLEGAL COMMAND: (((" << data.output() << ")))\n";
+		&& !is_legal_command(**command, player)) {
+			LOG_GAME << "ILLEGAL COMMAND in game: " << id_ << " ((("
+				<< (**command).output() << ")))\n";
 			std::stringstream msg;
-			msg << "Removing illegal command from: " << user->second.name()
+			msg << "Removing illegal command '" << (**command).first_child().to_string()
+				<< "' from: " << user->second.name()
 				<< ". Current player is: "
 				<< (player_info_->find(current_player()) != player_info_->end()
 					? player_info_->find(current_player())->second.name()
-					: "(unfound) ")// << nsides_ << "/" << end_turn_
-				<< ".\n";
-			LOG_GAME << msg.str();
+					: "(unfound)")
+				<< " (" << end_turn_ + 1 << "/" << nsides_ << ").";
+			LOG_GAME << msg.str() << " (socket: " << current_player()
+				<< ") (game id: " << id_ << ")\n";
 			send_and_record_server_message(msg.str().c_str());
-			//TODO: make an easy way to convert a simple wml node to a string
-			//LOG_GAME << (**command).output();
+
 			marked.push_back(index - marked.size());
 		} else if ((**command).child("speak")) {
 			simple_wml::node& speak = *(**command).child("speak");
@@ -876,7 +855,9 @@ bool game::end_turn() {
 	return true;
 }
 
-bool game::add_player(const network::connection player, bool observer, bool admin) {
+//@todo differentiate between "observers not allowed" and "player already in the game" errors.
+//      maybe return a string with an error message.
+bool game::add_player(const network::connection player, bool observer) {
 	if(is_member(player)) {
 		ERR_GAME << "ERROR: Player is already in this game. (socket: "
 			<< player << ")\n";
@@ -899,7 +880,7 @@ bool game::add_player(const network::connection player, bool observer, bool admi
 		DBG_GAME << "adding player...\n";
 		players_.push_back(player);
 		send_and_record_server_message((user->second.name() + " has joined the game.").c_str(), player);
-	} else if (!allow_observers() && !admin) {
+	} else if (!allow_observers() && !user->second.is_moderator()) {
 		return false;
 	} else {
 		if (!observer) became_observer = true;
@@ -913,17 +894,20 @@ bool game::add_player(const network::connection player, bool observer, bool admi
 		// Send observer join to everyone except the new observer.
 		send_data(observer_join, player);
 	}
+	LOG_GAME << network::ip_address(player) << "\t" << user->second.name()
+		<< "\tjoined game:\t\"" << name_ << "\" (" << id_ << ")"
+		<< (observer || became_observer ? " as an observer" : "")
+		<< ". (socket: " << player << ")\n";
 	user->second.mark_available(id_, name_);
 	DBG_GAME << debug_player_info();
 	// Send the user the game data.
 	//std::cerr << "SENDING LEVEL {{{" << level_.output() << "}}}\n";
-	simple_wml::string_span level_data = level_.output_compressed();
-	network::send_raw_data(level_data.begin(), level_data.size(), player,"game_level");
+	if (!send_to_one(level_, player)) return false;
+
 	if(started_) {
 		//tell this player that the game has started
 		static simple_wml::document start_game_doc("[start_game]\n[/start_game]\n", simple_wml::INIT_COMPRESSED);
-		static const simple_wml::string_span start_game = start_game_doc.output_compressed();
-		network::send_raw_data(start_game.begin(), start_game.size(), player,start_game_doc.root().first_child().to_string());
+		if (!send_to_one(start_game_doc, player)) return false;
 		// Send observer join of all the observers in the game to the new player
 		// only once the game started. The client forgets about it anyway
 		// otherwise.
@@ -946,7 +930,7 @@ bool game::add_player(const network::connection player, bool observer, bool admi
 	return true;
 }
 
-bool game::remove_player(const network::connection player, const bool disconnect) {
+bool game::remove_player(const network::connection player, const bool disconnect, const bool destruct) {
 	if (!is_member(player)) {
 		ERR_GAME << "ERROR: User is not in this game. (socket: "
 			<< player << ")\n";
@@ -971,26 +955,28 @@ bool game::remove_player(const network::connection player, const bool disconnect
 	const bool observer = is_observer(player);
 	players_.erase(std::remove(players_.begin(), players_.end(), player), players_.end());
 	observers_.erase(std::remove(observers_.begin(), observers_.end(), player), observers_.end());
-	const bool game_ended = (players_.empty() || (host && !started_));
+	const bool game_ended = players_.empty() || (host && !started_);
 	const player_map::iterator user = player_info_->find(player);
 	if (user == player_info_->end()) {
 		ERR_GAME << "ERROR: Could not find user in player_info_. (socket: "
 			<< player << ")\n";
-		return false;
+		return game_ended;
 	}
 	LOG_GAME << network::ip_address(user->first) << "\t" << user->second.name()
-		<< (game_ended ? (started_ ? "\tended" : "\taborted") : "\thas left")
+		<< ((game_ended && !(observer && destruct))
+			? (started_ ? "\tended" : "\taborted") : "\thas left")
 		<< " game:\t\"" << name_ << "\" (" << id_ << ")"
-		<< (game_ended && started_ ? " at turn: "
+		<< (game_ended && started_ && !(observer && destruct) ? " at turn: "
 			+ lexical_cast_default<std::string,size_t>(current_turn())
 			+ " with reason: '" + termination_reason() + "'" : "")
 		<< (observer ? " as an observer" : "")
 		<< (disconnect ? " and disconnected" : "")
 		<< ". (socket: " << user->first << ")\n";
-	if (game_ended) {
+	if (game_ended && started_ && !(observer && destruct)) {
 		send_server_message_to_all((user->second.name() + " ended the game.").c_str(), player);
-		return true;
 	}
+	if (game_ended || destruct) return game_ended;
+
 	// Don't mark_available() since the player got already removed from the
 	// games_and_users_list_.
 	if (!disconnect) user->second.mark_available();
@@ -1006,32 +992,35 @@ bool game::remove_player(const network::connection player, const bool disconnect
 		notify_new_host();
 	}
 
+	bool ai_transfer = false;
 	// Look for all sides the player controlled and drop them.
 	// (Give them to the host.)
 	for (side_vector::iterator side = sides_.begin(); side != sides_.end(); ++side)	{
 		size_t side_num = side - sides_.begin();
-		if (*side != player || side_controllers_[side_num] == "ai") continue;
+		if (*side != player) continue;
+		if (side_controllers_[side_num] == "ai") ai_transfer = true;
 
-		change_controller(side_num, player_info_->find(owner_));
+		player_map::const_iterator o = player_info_->find(owner_);
+		const std::string owner_name = (o != player_info_->end() ? o->second.name() : "(unknown)");
+		change_controller(side_num, owner_, owner_name);
 		// Check whether the host is actually a player and make him one if not.
 		if (!is_player(owner_)) {
 			DBG_GAME << "making the owner a player...\n";
 			observers_.erase(std::remove(observers_.begin(), observers_.end(), owner_), observers_.end());
 			players_.push_back(owner_);
-			send_observerquit(player_info_->find(owner_));
+			send_observerquit(o);
 		}
 
 		//send the host a notification of removal of this side
-		char side_drop_buf[64];
-		sprintf(side_drop_buf, "%d", static_cast<int>(side_num + 1));
+		const std::string side_drop = lexical_cast<std::string, size_t>(side_num + 1);
 		simple_wml::document drop;
-		drop.root().set_attr("side_drop", side_drop_buf);
-		// We're only dropping non-ai sides here. (The client just cares if it's an AI or not.)
-		drop.root().set_attr("controller", "network");
+		drop.root().set_attr("side_drop", side_drop.c_str());
+		drop.root().set_attr("controller", side_controllers_[side_num].c_str());
 
 		send_to_one(drop, owner_);
 	}
-	if (host) transfer_ai_sides();
+	if (ai_transfer) send_and_record_server_message("AI sides transferred to host.");
+
 	DBG_GAME << debug_player_info();
 
 	send_user_list(player);
@@ -1061,8 +1050,7 @@ void game::load_next_scenario(const player_map::const_iterator user) const {
 	send_server_message_to_all((user->second.name() + " advances to the next scenario").c_str(), user->first);
 	simple_wml::document cfg_scenario;
 	level_.root().copy_into(cfg_scenario.root().add_child("next_scenario"));
-	simple_wml::string_span data = cfg_scenario.output_compressed();
-	network::send_raw_data(data.begin(), data.size(), user->first, cfg_scenario.root().first_child().to_string());
+	if (!send_to_one(cfg_scenario, user->first)) return;
 	// Send the player the history of the game to-date.
 	send_history(user->first);
 	// Send observer join of all the observers in the game to the user.
@@ -1073,21 +1061,31 @@ void game::send_data(simple_wml::document& data, const network::connection exclu
 {
 	if (packet_type.empty())
 		packet_type = data.root().first_child().to_string();
-	simple_wml::string_span s = data.output_compressed();
-	const user_vector& users = all_game_users();
-	for(user_vector::const_iterator i = users.begin(); i != users.end(); ++i) {
-		if (*i != exclude) {
-			network::send_raw_data(s.begin(), s.size(), *i, packet_type);
+	try {
+		simple_wml::string_span s = data.output_compressed();
+		const user_vector& users = all_game_users();
+		for(user_vector::const_iterator i = users.begin(); i != users.end(); ++i) {
+			if (*i != exclude) {
+				network::send_raw_data(s.begin(), s.size(), *i, packet_type);
+			}
 		}
+	} catch (simple_wml::error& e) {
+		WRN_CONFIG << __func__ << ": simple_wml error: " << e.message << std::endl;
 	}
 }
 
-void game::send_to_one(simple_wml::document& data, const network::connection sock, std::string packet_type) const
+bool game::send_to_one(simple_wml::document& data, const network::connection sock, std::string packet_type) const
 {
 	if (packet_type.empty())
 		packet_type = data.root().first_child().to_string();
-	simple_wml::string_span s = data.output_compressed();
-	network::send_raw_data(s.begin(), s.size(), sock, packet_type);
+	try {
+		simple_wml::string_span s = data.output_compressed();
+		network::send_raw_data(s.begin(), s.size(), sock, packet_type);
+	} catch (simple_wml::error& e) {
+		WRN_CONFIG << __func__ << ": simple_wml error: " << e.message << std::endl;
+		return false;
+	}
+	return true;
 }
 
 void game::send_data_team(simple_wml::document& data,
@@ -1098,22 +1096,30 @@ void game::send_data_team(simple_wml::document& data,
 	DBG_GAME << __func__ << "...\n";
 	if (packet_type.empty())
 		packet_type = data.root().first_child().to_string();
-	simple_wml::string_span s = data.output_compressed();
-	for(user_vector::const_iterator i = players_.begin(); i != players_.end(); ++i) {
-		if(*i != exclude && is_on_team(team,*i)) {
-			network::send_raw_data(s.begin(), s.size(), *i, packet_type);
+	try {
+		simple_wml::string_span s = data.output_compressed();
+		for(user_vector::const_iterator i = players_.begin(); i != players_.end(); ++i) {
+			if(*i != exclude && is_on_team(team,*i)) {
+				network::send_raw_data(s.begin(), s.size(), *i, packet_type);
+			}
 		}
+	} catch (simple_wml::error& e) {
+		WRN_CONFIG << __func__ << ": simple_wml error: " << e.message << std::endl;
 	}
 }
 
 void game::send_data_observers(simple_wml::document& data, const network::connection exclude, std::string packet_type) const {
 	if (packet_type.empty())
 		packet_type = data.root().first_child().to_string();
-	simple_wml::string_span s = data.output_compressed();
-	for(user_vector::const_iterator i = observers_.begin(); i != observers_.end(); ++i) {
-		if (*i != exclude) {
-			network::send_raw_data(s.begin(), s.size(), *i, packet_type);
+	try {
+		simple_wml::string_span s = data.output_compressed();
+		for(user_vector::const_iterator i = observers_.begin(); i != observers_.end(); ++i) {
+			if (*i != exclude) {
+				network::send_raw_data(s.begin(), s.size(), *i, packet_type);
+			}
 		}
+	} catch (simple_wml::error& e) {
+		WRN_CONFIG << __func__ << ": simple_wml error: " << e.message << std::endl;
 	}
 }
 
@@ -1125,6 +1131,9 @@ bool game::is_on_team(const simple_wml::string_span& team, const network::connec
 				i != side_list.end(); ++i) {
 			if ((**i)["side"].to_int() != side - sides_.begin() + 1) continue;
 			if ((**i)["team_name"] != team) continue;
+			// Don't consider ai sides on a team.
+			if ((**i)["controller"] == "ai") continue;
+			if (side_controllers_[side - sides_.begin()] == "ai") continue;
 			DBG_GAME << "side: " << (**i)["side"].to_int() << " with team_name: " << (**i)["team_name"]
 			<< " belongs to player: " << player << std::endl;
 			return true;
@@ -1140,10 +1149,9 @@ std::string game::has_same_ip(const network::connection& user, bool observer) co
 	std::string clones;
 	for (user_vector::const_iterator i = users.begin(); i != users.end(); ++i) {
 		if (ip == network::ip_address(*i) && user != *i) {
-			if (i != users.begin()) clones += ", ";
 			const player_map::const_iterator pl = player_info_->find(*i);
 			if (pl != player_info_->end()) {
-				clones += pl->second.name();
+				clones += (clones.empty() ? "" : ", ") + pl->second.name();
 			}
 		}
 	}
@@ -1168,8 +1176,7 @@ void game::send_observerjoins(const network::connection sock) const {
 			send_data(cfg, *ob);
 		} else {
 			// Send to the (new) user.
-			const simple_wml::string_span& data = cfg.output_compressed();
-			network::send_raw_data(data.begin(), data.size(), sock);
+			send_to_one(cfg, sock);
 		}
 	}
 }
@@ -1202,12 +1209,74 @@ void game::send_history(const network::connection sock) const
 		delete *i;
 	}
 
-	simple_wml::document* doc = new simple_wml::document(buf.c_str(), simple_wml::INIT_STATIC);
-	const simple_wml::string_span& data = doc->output_compressed();
-	doc->compress();
-	network::send_raw_data(data.begin(), data.size(), sock,"game_history");
+	try {
+		simple_wml::document* doc = new simple_wml::document(buf.c_str(), simple_wml::INIT_STATIC);
+		const simple_wml::string_span& data = doc->output_compressed();
+		doc->compress();
+		network::send_raw_data(data.begin(), data.size(), sock,"game_history");
+		history_.clear();
+		history_.push_back(doc);
+	} catch (simple_wml::error& e) {
+		WRN_CONFIG << __func__ << ": simple_wml error: " << e.message << std::endl;
+	}
+
+}
+
+static bool is_invalid_filename_char(char c) {
+	return !(isalnum(c) || (c == '_') || (c == '-') || (c == '.')
+			|| (c == '(') || (c == ')') || (c == '#') || (c == ',')
+			|| (c == '!') || (c == '?') || (c == '^') || (c == '+')
+			|| (c == '*') || (c == ':') || (c == '=') || (c == '@')
+			|| (c == '%') || (c == '\''));
+}
+
+void game::save_replay() {
+	if (!save_replays_ || !started_ || history_.empty()) return;
+
+	std::string replay_commands;
+	for(std::vector<simple_wml::document*>::iterator i = history_.begin();
+			i != history_.end(); ++i) {
+		const simple_wml::node::child_list& turn_list = (*i)->root().children("turn");
+		for (simple_wml::node::child_list::const_iterator turn = turn_list.begin();
+				turn != turn_list.end(); ++turn) {
+			replay_commands += (*turn)->output();
+		}
+		delete *i;
+	}
 	history_.clear();
-	history_.push_back(doc);
+
+	std::stringstream name;
+	name << level_["name"] << " Turn " << current_turn();
+
+	std::stringstream replay_data;
+	replay_data << "campaign_type=\"multiplayer\"\n"
+	<< "difficulty=\"NORMAL\"\n"
+	<< "label=\"" << name.str() << "\"\n"
+	<< "mp_game_title=\"" << name_ << "\"\n"
+	<< "random_seed=\"" << level_["random_seed"] << "\"\n"
+	<< "version=\"" << level_["version"] << "\"\n"
+	<< "[replay]\n" << replay_commands << "[/replay]\n"
+	<< "[replay_start]\n" << level_.output() << "[/replay_start]\n";
+
+	name << " (" << id_ << ").gz";
+
+	std::string replay_data_str = replay_data.str();
+	try {
+		simple_wml::document replay(replay_data_str.c_str(), simple_wml::INIT_STATIC);
+
+		std::string filename(name.str());
+		std::replace(filename.begin(), filename.end(), ' ', '_');
+		filename.erase(std::remove_if(filename.begin(), filename.end(), is_invalid_filename_char), filename.end());
+		DBG_GAME << "saving replay: " << filename << std::endl;
+		scoped_ostream os(ostream_file(replay_save_path_ + filename));
+		(*os) << replay.output_compressed();
+
+		if (!os->good()) {
+			ERR_GAME << "Could not save replay! (" << filename << ")\n";
+		}
+	} catch (simple_wml::error& e) {
+		WRN_CONFIG << __func__ << ": simple_wml error: " << e.message << std::endl;
+	}
 }
 
 void game::record_data(simple_wml::document* data) {
@@ -1215,6 +1284,14 @@ void game::record_data(simple_wml::document* data) {
 	history_.push_back(data);
 }
 
+void game::clear_history() {
+	if (history_.empty()) return;
+	for(std::vector<simple_wml::document*>::iterator i = history_.begin(); i != history_.end(); ++i) {
+		delete *i;
+	}
+	history_.clear();
+}
+
 void game::set_description(simple_wml::node* desc) {
 	description_ = desc;
 	if(!password_.empty()) {
@@ -1223,13 +1300,13 @@ void game::set_description(simple_wml::node* desc) {
 }
 
 void game::set_termination_reason(const std::string& reason) {
-	if (reason == "out of sync") {
+/*	if (reason == "out of sync") {
 		simple_wml::string_span era;
 		if (level_.child("era")) {
 			era = level_.child("era")->attr("id");
 		}
 		termination_ = "out of sync - " + era.to_string();
-	}
+	}*/
 	if (termination_.empty()) { termination_ = reason; }
 }
 
