@@ -1,6 +1,6 @@
-/* $Id: team.hpp 24034 2008-02-24 22:58:45Z jhinrichs $ */
+/* $Id: team.hpp 33039 2009-02-23 03:38:54Z sapient $ */
 /*
-   Copyright (C) 2003 - 2008 by David White <dave@whitevine.net>
+   Copyright (C) 2003 - 2009 by David White <dave@whitevine.net>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
 
    This program is free software; you can redistribute it and/or modify
@@ -17,8 +17,10 @@
 #include "config.hpp"
 #include "color_range.hpp"
 #include "game_config.hpp"
-#include "map.hpp"
+#include "map_location.hpp"
+#include "viewpoint.hpp"
 
+class gamemap;
 struct time_of_day;
 
 #include <set>
@@ -46,6 +48,7 @@ class team : public viewpoint
 
 		std::string write() const;
 		void read(const std::string& shroud_data);
+		void merge(const std::string& shroud_data);
 
 		bool enabled() const { return enabled_; }
 		void set_enabled(bool enabled) { enabled_ = enabled; }
@@ -71,12 +74,14 @@ public:
 		std::string start_gold;
 		std::string income;
 		int income_per_village;
+		size_t average_price;
+		float number_of_possible_recruits_to_force_recruit;
 		std::set<std::string> can_recruit;
 		std::vector<std::string> global_recruitment_pattern;
 		std::vector<std::string> recruitment_pattern;
 		std::vector<int> enemies;
 		std::string team_name;
-		std::string user_team_name;
+		t_string user_team_name;
 		std::string save_id;
 		// 'id' of the current player (not necessarily unique)
 		std::string current_player;
@@ -95,9 +100,8 @@ public:
 		 * displayed to the user. */
 		bool objectives_changed;
 
-		enum CONTROLLER { HUMAN, AI, NETWORK, EMPTY };
+		enum CONTROLLER { HUMAN, HUMAN_AI, AI, NETWORK, NETWORK_AI, EMPTY };
 		CONTROLLER controller;
-		bool persistent;
 		std::string ai_algorithm;
 
 		std::vector<config> ai_params;
@@ -105,6 +109,8 @@ public:
 
 		int villages_per_scout;
 		double leader_value, village_value;
+		//cached values for ai parameters
+		double aggression_, caution_;
 
 		std::vector<target> targets;
 
@@ -112,6 +118,7 @@ public:
 		bool disallow_observers;
 		bool allow_player;
 		bool no_leader;
+		bool hidden;
 
 		std::string music;
 
@@ -120,17 +127,17 @@ public:
 
 	static std::map<int, color_range> team_color_range_;
 	static const int default_team_gold;
-	team(const config& cfg, int gold=default_team_gold);
+	team(const config& cfg, const gamemap& map, int gold=default_team_gold);
 
 	~team() {};
 
 	void write(config& cfg) const;
 
-	bool get_village(const gamemap::location&);
-	void lose_village(const gamemap::location&);
+	bool get_village(const map_location&);
+	void lose_village(const map_location&);
 	void clear_villages() { villages_.clear(); }
-	const std::set<gamemap::location>& villages() const { return villages_; }
-	bool owns_village(const gamemap::location& loc) const
+	const std::set<map_location>& villages() const { return villages_; }
+	bool owns_village(const map_location& loc) const
 		{ return villages_.count(loc) > 0; }
 
 	int gold() const { return gold_; }
@@ -155,11 +162,26 @@ public:
 	void set_current_player(const std::string player)
 		{ info_.current_player = player; }
 
+	size_t average_recruit_price();
+	float num_pos_recruits_to_force()
+	{ return info_.number_of_possible_recruits_to_force_recruit; }
 	const std::set<std::string>& recruits() const
 		{ return info_.can_recruit; }
-	std::set<std::string>& recruits() { return info_.can_recruit; }
+	void add_recruits(const std::set<std::string>& recruits);
+	void remove_recruit(const std::string& recruits);
+	void set_recruits(const std::set<std::string>& recruits);
 	const std::vector<std::string>& recruitment_pattern() const
 		{ return info_.recruitment_pattern; }
+	bool remove_recruitment_pattern_entry(const std::string& key)
+	{
+	   	std::vector<std::string>::iterator itor = std::find(info_.recruitment_pattern.begin(),
+				info_.recruitment_pattern.end(),
+				key);
+		if (itor == info_.recruitment_pattern.end())
+			return false;
+		info_.recruitment_pattern.erase(itor);
+		return true;
+	}
 	const std::string& name() const
 		{ return info_.name; }
 	const std::string& save_id() const { return info_.save_id; }
@@ -195,19 +217,24 @@ public:
 		seen_[index] = true;
 	}
 
-	double aggression() const { return aggression_; }
-	double caution() const { return caution_; }
+	double aggression() const { return info_.aggression_; }
+	double caution() const { return info_.caution_; }
 
 	team_info::CONTROLLER controller() const { return info_.controller; }
 	bool is_human() const { return info_.controller == team_info::HUMAN; }
-	bool is_network() const { return info_.controller == team_info::NETWORK; }
-	bool is_ai() const { return info_.controller == team_info::AI; }
+	bool is_human_ai() const { return info_.controller == team_info::HUMAN_AI; }
+	bool is_network_human() const { return info_.controller == team_info::NETWORK; }
+	bool is_network_ai() const { return info_.controller == team_info::NETWORK_AI; }
+	bool is_ai() const { return info_.controller == team_info::AI || is_human_ai(); }
 	bool is_empty() const { return info_.controller == team_info::EMPTY; }
 
-	bool is_persistent() const { return info_.persistent; }
+	bool is_local() const { return is_human() || is_human_ai(); }
+	bool is_network() const { return is_network_human() || is_network_ai(); }
 
 	void make_human() { info_.controller = team_info::HUMAN; }
+	void make_human_ai() { info_.controller = team_info::HUMAN_AI; }
 	void make_network() { info_.controller = team_info::NETWORK; }
+	void make_network_ai() { info_.controller = team_info::NETWORK_AI; }
 	void make_ai() { info_.controller = team_info::AI; }
 	// Should make the above make_*() functions obsolete, as it accepts controller
 	// by lexical or numerical id
@@ -226,6 +253,7 @@ public:
 	const config& ai_parameters() const { return aiparams_; }
 	const config& ai_memory() const { return info_.ai_memory_; }
 	void set_ai_memory(const config& ai_mem);
+	void set_ai_parameters(const config::child_list& ai_parameters);
 
 	double leader_value() const { return info_.leader_value; }
 	void set_leader_value(double value) { info_.leader_value = value; }
@@ -238,35 +266,41 @@ public:
 
 	//Returns true if the hex is shrouded/fogged for this side, or
 	//any other ally with shared vision.
-	bool shrouded(const gamemap::location& loc) const;
-	bool fogged(const gamemap::location& loc) const;
+	bool shrouded(const map_location& loc) const;
+	bool fogged(const map_location& loc) const;
 
 	bool uses_shroud() const { return shroud_.enabled(); }
 	bool uses_fog() const { return fog_.enabled(); }
 	bool fog_or_shroud() const { return uses_shroud() || uses_fog(); }
-	bool clear_shroud(const gamemap::location& loc) { return shroud_.clear(loc.x+1,loc.y+1); }
-	void place_shroud(const gamemap::location& loc) { shroud_.place(loc.x+1,loc.y+1); }
-	bool clear_fog(const gamemap::location& loc) { return fog_.clear(loc.x+1,loc.y+1); }
+	bool clear_shroud(const map_location& loc) { return shroud_.clear(loc.x+1,loc.y+1); }
+	void place_shroud(const map_location& loc) { shroud_.place(loc.x+1,loc.y+1); }
+	bool clear_fog(const map_location& loc) { return fog_.clear(loc.x+1,loc.y+1); }
 	void refog() { fog_.reset(); }
 	void set_shroud(bool shroud) { shroud_.set_enabled(shroud); }
 	void set_fog(bool fog) { fog_.set_enabled(fog); }
 
-	bool knows_about_team(size_t index) const;
+	/** Merge a WML shroud map with the shroud data of this player. */
+	void merge_shroud_map_data(const std::string& shroud_data);
+
+	bool knows_about_team(size_t index, bool is_multiplayer) const;
 	bool copy_ally_shroud();
 
 	bool auto_shroud_updates() const { return auto_shroud_updates_; }
 	void set_auto_shroud_updates(bool value) { auto_shroud_updates_ = value; }
 	bool get_disallow_observers() {return info_.disallow_observers; };
 	std::string map_colour_to() const { return info_.colour; };
-	bool& no_leader()  { return info_.no_leader; }
+	bool no_leader() const { return info_.no_leader; }
+	void have_leader(bool value=true) { info_.no_leader = !value; }
+	bool hidden() const { return info_.hidden; }
+	void set_hidden(bool value) { info_.hidden=value; }
 
 	static int nteams();
 
 	//function which, when given a 1-based side will return the colour used by that side.
 	static const color_range get_side_color_range(int side);
-	static const Uint32 get_side_rgb(int side) { return(get_side_color_range(side).mid()); }
-	static const Uint32 get_side_rgb_max(int side) { return(get_side_color_range(side).max()); }
-	static const Uint32 get_side_rgb_min(int side) { return(get_side_color_range(side).min()); }
+	static Uint32 get_side_rgb(int side) { return(get_side_color_range(side).mid()); }
+	static Uint32 get_side_rgb_max(int side) { return(get_side_color_range(side).max()); }
+	static Uint32 get_side_rgb_min(int side) { return(get_side_color_range(side).min()); }
 	static const SDL_Color get_minimap_colour(int side);
 	static std::string get_side_colour_index(int side);
 	static std::string get_side_highlight(int side);
@@ -282,7 +316,7 @@ private:
 	const std::vector<const shroud_map*>& ally_fog(const std::vector<team>& teams) const;
 
 	int gold_;
-	std::set<gamemap::location> villages_;
+	std::set<map_location> villages_;
 
 	shroud_map shroud_, fog_;
 
@@ -295,8 +329,6 @@ private:
 
 	config aiparams_;
 
-	//cached values for ai parameters
-	double aggression_, caution_;
 
 	bool calculate_enemies(size_t index) const;
 	bool calculate_is_enemy(size_t index) const;
@@ -312,10 +344,11 @@ struct teams_manager {
 	~teams_manager();
 
 	bool is_observer();
+	static const std::vector<team>& get_teams();
 };
 
 namespace player_teams {
-	int village_owner(const gamemap::location& loc);
+	int village_owner(const map_location& loc);
 }
 
 bool is_observer();
