@@ -1,4 +1,4 @@
-/* $Id: unit_frame.cpp 33743 2009-03-17 03:53:42Z alink $ */
+/* $Id: unit_frame.cpp 35578 2009-05-11 21:45:03Z boucman $ */
 /*
    Copyright (C) 2006 - 2009 by Jeremy Rosen <jeremy.rosen@enst-bretagne.fr>
    Part of the Battle for Wesnoth Project http://www.wesnoth.org/
@@ -156,8 +156,8 @@ template class progressive_<double>;
 
 
 frame_builder::frame_builder(const config& cfg,const std::string& frame_string) :
-	image_(image::locator()),
-	image_diagonal_(image::locator()),
+	image_(),
+	image_diagonal_(),
 	image_mod_(""),
 	halo_(""),
 	halo_x_(""),
@@ -465,13 +465,16 @@ bool unit_frame::invalidate(const bool force,const int frame_time,const map_loca
 	const frame_parameters current_data = merge_parameters(frame_time,animation_val,engine_val,primary);
 	double tmp_offset = current_data.offset;
 	int d2 = game_display::get_singleton()->hex_size() / 2;
+	bool image_fit_hex = false;
 
 	image::locator image_loc;
 	if(direction != map_location::NORTH && direction != map_location::SOUTH) {
 		image_loc = current_data.image_diagonal;
+		image_fit_hex = image::is_in_hex(image_loc);
 	}
 	if(image_loc.is_void() || image_loc.get_filename() == "") { // invalid diag image, or not diagonal
 		image_loc = current_data.image;
+		image_fit_hex = image::is_in_hex(image_loc);
 	}
 
 	surface image;
@@ -490,14 +493,22 @@ bool unit_frame::invalidate(const bool force,const int frame_time,const map_loca
 	if (image != NULL) {
 		const int x = static_cast<int>(tmp_offset * xdst + (1.0-tmp_offset) * xsrc)+current_data.x+d2-(image->w/2);
 		const int y = static_cast<int>(tmp_offset * ydst + (1.0-tmp_offset) * ysrc)+current_data.y+d2-(image->h/2);
+		const SDL_Rect r = {x,y,image->w,image->h};
+		// check if the unit fit in a hex
+		bool in_hex = image_fit_hex && r.x==xsrc && r.y==ysrc
+				&& r.w==disp->hex_size() && r.h==disp->hex_size();
+		// check if our underlying hexes are invalidated
+		bool rect_need_update = in_hex ?
+				disp->hex_need_update(src) : disp->rectangle_need_update(r);
 		// if we need to update ourselve because we changed, invalidate our hexes
 		// and return whether or not our hexs was invalidated
-		const SDL_Rect r = {x,y,image->w,image->h};
-		if(force || need_update() || disp->rectangle_need_update(r)){
+		if(force || need_update() || rect_need_update) {
 			// invalidate ouself to be called at redraw time
 			result |= disp->invalidate(src);
-			// invalidate all hex we plan to overwrite
-			result |= disp->invalidate_visible_locations_in_rect(r);
+			if(in_hex == false) {
+				// invalidate all hexes we plan to overwrite
+				result |= disp->invalidate_visible_locations_in_rect(r);
+			}
 		}
 	} else {
 		// we have no "redraw surface" but we still need to invalidate our own hex
